# Contributing to `coverde`

**First off, thanks for taking the time to contribute!**

If you'd like to report a bug or join in the development of `coverde`, then here are some notes on how to do that.

<!-- Please **note** we have a [code of conduct](https://github.com/mrverdant13/coverde/blob/master/CODE_OF_CONDUCT.md), please follow it in all your interactions with the project. -->

## Contents

- [Reporting bugs and opening issues](#reporting-bugs-and-opening-issues)
- [Coding Guidelines](#coding-guidelines)
  - [Pre-requisites](#pre-requisites)
  - [Pull Requests](#pull-requests)

## Reporting bugs and opening issues

If you'd like a report a bug or open an issue then please:

**Check if there is an existing issue.** If that is the case, then please provide more information or give it a 👍.

When submitting an issue please describe the issue as clearly as possible, including how to
reproduce the bug, which situations it appears in, what you expected to happen, and what actually happens.

If you can include screenshots, trace data or any additional context, it could be quite helpful.

## Coding Guidelines

### Pre-requisites

- [Dart SDK][dart_link] - min version: 2.14.4 (stable)
- [Melos][melos_link] - min version: 0.4.11+2
- `coverde` - version: local (installed by executing `dart pub global activate --source path .` in the local `coverde` root folder)

### Pull Requests

**If you've got an idea for a change then please discuss it by opening an issue.**

We love pull requests, so be bold with them!

For commits, this repo follows the [Conventional Commits Specification][conventional_commit_specification_link]. Please, check its set of rules.

**Before submitting a PR, consider the following requirements**:

- PRs should include formatted Dart code.
- PRs should not include info nor warning lint highlights.
- PRs should include the latest files generated by the `build_runner` tool.
- PRs should keep a resultant codebase test coverage of 100% (without considering generated files).
- PRs should keep a resultant pub score with a maximum tolerance of 10.

To easily check if these conditions are satisfied, you can use a collection of melos scripts defined in the [melos.yaml](melos.yaml) file.

**To run the entire overhaul workflow according to requirements, run `melos run WF`**

[conventional_commit_specification_link]: https://www.conventionalcommits.org/en/v1.0.0/
[dart_link]: https://dart.dev/
[melos_link]: https://github.com/invertase/melos
